#lang typed/racket

(require "monascheme-core.rkt")
(require "test.rkt")
(require "common-types.rkt")

(provide run-test-la-exceptions)

(: lifted_write (Any -> (EL Any)))
(define lifted_write
  (lambda: ([s : Any]) (fmap write (return s))))

;; Trace advice
(: trace ((Any -> (EL Any)) -> (Any -> (EL Any))))
(define (trace proceed)  
  (lambda: ([a : Any])  
           (do
            (lifted_write 'a)     
            (proceed a))))

(define (raise_aspect proceed)
  (lambda (a) (raise* a)))

(define (raise_false f) (raise* "false"))

(: try_aspect ((Any -> (EL Any)) -> (Any -> (EL Any))))
(define (try_aspect proceed)
  (lambda: ([a : Any])
           (try-with (proceed a) (return 1))))

;; An advice can place an handler at level 0 to catch
;; exceptions of the base computation
(: conflating_aspect ((Any -> (EL Any)) -> (Any -> (EL Any))))
(define (conflating_aspect proceed)
  (lambda: ([a : Any])
           (down (try-with (up (proceed a)) (return "catched_adv")))))

(define (run-test-la-exceptions)
  
  (test/equal
   (run 
    (deploy-fluid (call lifted_write)
                  (return raise_aspect)
                  (lifted_write 'hello)))
   (app/prim cons 'Exception 1))
  
  (test/equal
   (run 
    (deploy-fluid (call lifted_write)
                  (return raise_aspect)
                  (try-with (lifted_write 'hello) (return 1))))
   (app/prim cons 'Exception 1))
  
  (test/equal
   (run 
    (deploy-fluid (call raise_false)               
                  (return try_aspect)
                  (raise_false '())))
   (app/prim cons 'Exception 0))
  
  (test/equal
   (run 
    (deploy-fluid (call raise_false)               
                  (return conflating_aspect)
                  (raise_false '())))
   "catched_adv")
  
  (test/equal
   (run 
    (deploy-fluid (call lifted_write)               
                  (return raise_aspect)
                  (up (try-with  (down (lifted_write 'hello)) (return "catched_base")))))
   "catched_base")
  
  (test/equal
   (run (try-with (raise_false 1) (return "hello")))
   "hello")
  
  (test/equal
   (run (deploy-fluid (call raise_false) (return trace) (try-with (raise_false 1) (return "hello"))))
   "hello")
  
  (test/equal
   (run (deploy-fluid (call lifted_write) (return trace) (try-with (raise_false 1) (return "hello"))))
   "hello"))
